/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.dialogs;

import java.util.ArrayList;
import java.util.Map;
import java.util.SortedSet;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ReportTemplateDescriptor;
import com.unitesk.requality.eclipse.editors.panels.SelectFilePanel;
import com.unitesk.requality.eclipse.editors.panels.SelectRequirementPanel;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;

public class ReportGenDialog extends Dialog {
    private static String TITLE = "Report Generator";
    private static Image ICON = PlatformUI.getWorkbench().getSharedImages()
            .getImage(ISharedImages.IMG_OBJ_ELEMENT);

    private static String LBL_ROOT = "Root requirement:";
    private static String LBL_TEMPLATE = "Template:";
    private static String LBL_SAVE = "Save to:";

    private Requirement req = null;
    private SelectRequirementPanel rootReqPanel;
    private Combo templates;
    private int selectTplCount = 0;
    private String savePath = "";
    private SelectFilePanel outFolderPanel;
    private ArrayList<ReportTemplateDescriptor> tpls = new ArrayList<ReportTemplateDescriptor>();
    private TreeDB db = null;

    private Listener dataChangeListener = new Listener() {
        @Override
        public void handleEvent(Event event) {
            updateUI();
            savePath = outFolderPanel.getSelectedPath();
        }
    };

    public ReportGenDialog(Shell shell, TreeDB db) {
        super(shell);
        this.db = db;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(2, false));
        GridData layData = new GridData(GridData.FILL_BOTH);
        layData.widthHint = 700;
        container.setLayoutData(layData);

        new Label(container, SWT.NONE).setText(LBL_ROOT);
        rootReqPanel = new SelectRequirementPanel(container, SWT.NONE, db);
        rootReqPanel.setCanSelectRoot(true);
        if(req==null) {
            req = (Requirement) db.getNode(Requirement
                    .getTypeRootQId());
        }
        rootReqPanel.setRequirement(req);
        rootReqPanel.addListener(SWT.Modify, dataChangeListener);

        new Label(container, SWT.NONE).setText(LBL_TEMPLATE);
        createSelectTemplatePanel(container);
        templates.addListener(SWT.Modify, dataChangeListener);

        IFolder repFolder = TreesTracker.getTracker().getProject(db)
                .getFolder("reports");
        if (!repFolder.exists()) {
            try {
                repFolder.create(true, true, null);
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }

        new Label(container, SWT.NONE).setText(LBL_SAVE);
        savePath = repFolder.getLocation().toOSString();
        outFolderPanel = new SelectFilePanel(container, SWT.NONE, savePath);
        outFolderPanel.setSelectOnlyFolder(true);
        outFolderPanel.addListener(SWT.Modify, dataChangeListener);

        this.getShell().setText(TITLE);
        this.getShell().setImage(ICON);
        return container;
    }

    private Composite createSelectTemplatePanel(Composite root) {
        Composite panel = new Composite(root, SWT.NONE);
        panel.setLayout(new GridLayout());
        templates = new Combo(panel, SWT.NONE | SWT.READ_ONLY);
        SortedSet<ReportTemplateDescriptor> tplSet = Activator.getDefault()
                .getReportTemplates();
        for (ReportTemplateDescriptor tpl : tplSet) {
            tpls.add(tpl);
            templates.add(tpl.getName());
        }
        templates.select(0);
        return panel;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        super.createButtonsForButtonBar(parent);
        updateUI();
    }

    private void updateUI() {
        Button okButton = this.getButton(IDialogConstants.OK_ID);
        okButton.setEnabled(isDataCorrect());
        selectTplCount = templates.getSelectionIndex();
    }

    private boolean isDataCorrect() {
        if (rootReqPanel.getSelectedRequirement() == null)
            return false;
        if (tpls.size() == 0)
            return false;
        return true;
    }

    public Map<String, Object> getData() {
        Map<String, Object> data = tpls.get(selectTplCount).getProcessor()
                .getData();
        data.put("rootReqQId", rootReqPanel.getSelectedRequirement().getQualifiedId());
        return data;
    }

    public String getTemplate() {
        return tpls.get(selectTplCount).getProcessor().getTemplatePath();
    }

    public String getSaveDir() {
        return savePath;
    }

    public void setStartNode(Requirement req) {
        this.req = req;
    }
}
