/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.editors.browser;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.dialogs.SelectNodeDialog;
import com.unitesk.requality.eclipse.dialogs.SelectReqDialog;
import com.unitesk.requality.nodetypes.Requirement;

public class AddSelectionToRequirement extends Action {
    protected ReqMarker reqMarker;
    private Requirement target;
    protected boolean needSelectReq;
    private String nextId;

    public AddSelectionToRequirement(ReqMarker reqMarker) {
        super("Add to Requirement...");
        this.reqMarker = reqMarker;
        needSelectReq = true;
    }

    public AddSelectionToRequirement(ReqMarker reqMarker, Requirement target) {
        super("Add to Requirement: " + target.getName());
        this.reqMarker = reqMarker;
        needSelectReq = false;
        this.target = target;
    }

    public AddSelectionToRequirement(ReqMarker reqMarker, Requirement target,
            String nextId) {
        super("Create new Requirement: "
                + (!target.getQualifiedId()
                        .equals(Requirement.getTypeRootQId()) ? target
                        .getName() + "/" : "") + nextId);
        this.reqMarker = reqMarker;
        this.nextId = nextId;
        needSelectReq = false;
        this.target = target;
    }

    @Override
    public boolean isEnabled() {
        try {
            return reqMarker.getBrowser().isSelectable();
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public void run() {
        // TODO: signal if not enable
        if (!isEnabled())
            return;

        if (target != null && !needSelectReq) {
            if (nextId != null) {
                Requirement req = (Requirement) target.createChild(nextId);
                reqMarker.addSelectionTo(req);
            } else {
                reqMarker.addSelectionTo(target);
            }
            return;
        }

        SelectNodeDialog reqSelectDlg = new SelectReqDialog(PlatformUI
                .getWorkbench().getActiveWorkbenchWindow().getShell(),
                reqMarker.getTreeDB());
        if(target!=null) {
            reqSelectDlg.select(target);
        }
        reqSelectDlg.setCanSelectRoot(false);
        if (reqSelectDlg.open() == SelectNodeDialog.OK) {
            Activator.getDefault().setActiveNode(reqSelectDlg.getResult(),
                    reqSelectDlg.getResult().getType());
            reqMarker.addSelectionTo((Requirement) reqSelectDlg.getResult());
        }
        reqMarker.setFocus();
    }

    private static String getNumberStringFromId(String in) {
        Pattern pattern = Pattern.compile("(.*\\D|)(\\d*)$");
        Matcher m = pattern.matcher(in);
        if (m.matches()) {
            return m.group(m.groupCount());
        } else {
            return null;
        }
    }

    public static String getNextId(TreeNode parent) {
        String newId = "00";
        String fullNewId = "00";

        for (int i = 0; i < parent.sizeChildren(); i++) {
            String number = getNumberStringFromId(parent.getChild(i).getId());
            if (number == null) {
                continue;
            }
            try {
                if (Integer.parseInt(number) > Integer.parseInt(newId)) {
                    fullNewId = parent.getChild(i).getId();
                    newId = number;
                }
            } catch (NumberFormatException e) {
            }
        }

        return incLastNumberInString(fullNewId);
    }

    // Utilites
    public static String incLastNumberInString(String str) {
        String regex = "0*\\d*";
        String tmp = str;

        while (!tmp.matches(regex))
            tmp = tmp.substring(1);
        String fixture = str.substring(0, str.length() - tmp.length());
        String tail = String.valueOf(Integer.parseInt(tmp) + 1);

        if (tail.length() < tmp.length()) {
            tail = makeZeroString(tmp.length() - tail.length()) + tail;
        }
        return fixture + tail;
    }

    private static String makeZeroString(int zeroCount) {
        StringBuilder sb = new StringBuilder(zeroCount);
        for (int i = 0; i < zeroCount; i++)
            sb.append("0");
        return sb.toString();
    }
}
