package com.unitesk.requality.eclipse.editors.browser;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.IContributedContentsView;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.eclipse.views.CNFLabelProvider;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;

public class ReqMarkerOutlinePage extends ContentOutlinePage implements
        ITabbedPropertySheetPageContributor, IAdaptable {
    private Document input;
    private ISelectionListener locationSelectionListener = new ISelectionListener() {

        @Override
        public void selectionChanged(IWorkbenchPart part, ISelection selection) {
            if (!(selection instanceof IStructuredSelection)) {
                return;
            }
            if(getTreeViewer().getSelection().equals(selection)) {
                return;
            }
            IStructuredSelection sel = (IStructuredSelection) selection;
// TODO: remove
//            System.out.println("selection changed from "+getTreeViewer().getSelection()+" to "+selection);
            if (!(sel.getFirstElement() instanceof Location)) {
                return;
            }
            Location loc = (Location) sel.getFirstElement();
            getTreeViewer().setSelection(new StructuredSelection(loc));
        }
    };

    @Override
    public void createControl(Composite parent) {
        super.createControl(parent);
        getTreeViewer().setContentProvider(
                new ReqMarkerOutlineContentProvider());
        getTreeViewer().setSorter(new ViewerSorter());
        getTreeViewer().setLabelProvider(new CNFLabelProvider());
        getTreeViewer().setInput(input);
        getSite().getWorkbenchWindow().getSelectionService()
                .addPostSelectionListener(locationSelectionListener);
    }

    public void setInput(Object input) {
        this.input = (Document) input;
    }

    public void refresh() {
        getTreeViewer().refresh();
    }

    @Override
    public void dispose() {
        super.dispose();
        getSite().getWorkbenchWindow().getSelectionService()
                .removePostSelectionListener(locationSelectionListener);
    }

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequirementPropertyContributor";
    }

    @SuppressWarnings("rawtypes")
    @Override
    public Object getAdapter(Class required) {
        if (required == IContributedContentsView.class) {
            return new IContributedContentsView() {
                public IWorkbenchPart getContributingPart() {
                    return PlatformUI
                            .getWorkbench()
                            .getActiveWorkbenchWindow()
                            .getActivePage()
                            .findView(
                                    "com.unitesk.requality.views.RequalityExplorer");
                }
            };

        }
        if (required == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(this);
        }
        return Platform.getAdapterManager().getAdapter(this, required);
    }
}
