package com.unitesk.requality.eclipse.handlers.document;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.views.RequalityCNF;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class RemoveDocumentHandler extends AbstractHandler {
    private static final String REMOVE_DOCUMENT = "Remove Document";
    private static final String REMOVE_FOLDER = "Remove Folder";

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode node = RequalityCNF.getSelectedNode();
        if (node instanceof DocFolder) {
            recDelNode(node);
        }
        if (node instanceof Document) {
            removeDocument((Document) node);
        }
        return null;
    }

    @Override
    public boolean isEnabled() {
        TreeNode node = RequalityCNF.getSelectedNode();
        if (node == null) {
            return false;
        }
        if (node.getQualifiedId().equals(Document.getTypeRootQId())) {
            return false;
        }
        if (!(node.getType().equals("Document") || node.getType().equals(
                "DocFolder"))) {
            return false;
        }
        return true;
    }

    private void recDelNode(TreeNode node) {
        if (node instanceof Document) {
            removeDocument((Document) node);
        }

        if (node instanceof DocFolder) {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), REMOVE_FOLDER,
                    "Delete selected document folder?")) {
                return;
            }
            if (folderHasDocumentWithLocation((DocFolder) node)) {
                if (!MessageDialog
                        .openConfirm(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(),
                                REMOVE_FOLDER,
                                "Folder contains document with marked requirements, continue?")) {
                    return;
                }
            }

            for (int i = 0; i < node.sizeChildren(); i++) {
                recDelNode(node.getChild(i));
            }
            node.delete();
        }
    }

    private void removeDocument(Document doc) {

        if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getShell(), REMOVE_DOCUMENT,
                "Delete selected document?")) {
            return;
        }

        if (doc.sizeChildren() > 0) {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), REMOVE_FOLDER,
                    "Document contains marked requirements, continue?")) {
                return;
            }
        }
        removeLocationsFromRequirementd(doc);
        doc.delete();
    }

    private boolean folderHasDocumentWithLocation(DocFolder folder) {
        if (folder.sizeChildren() == 0) {
            return false;
        }

        for (int i = 0; i < folder.sizeChildren(); i++) {
            TreeNode node = folder.getChild(i);
            if (node instanceof DocFolder) {
                if (folderHasDocumentWithLocation((DocFolder) node)) {
                    return true;
                }
            }
            if (node instanceof Document) {
                Document doc = (Document) node;
                if (doc.sizeLocations() > 0) {
                    return true;
                }
            }
        }
        return false;
    }
    private void removeLocationsFromRequirementd(Document doc) {
        TreeDB db = doc.getTreeDB();
        Set<String> locQIds = new HashSet<String>();
        for(int i=0; i < doc.sizeChildren(); i++) {
            locQIds.add(doc.getChild(i).getQualifiedId());
        }
        
        for (TreeNode node : new DeepFirstTreeWalker(db.getNode(Requirement
                .getTypeRootQId()))) {

            if(!node.getType().equals("Requirement")) {
                continue;
            }

            ArrayList<Location> removeLocations = new ArrayList<Location>();
            Requirement req = (Requirement) node;

            for (Location loc : req.getLocations()) {
                if (locQIds.contains(loc.getQualifiedId())) {
                    removeLocations.add(loc);
                }
            }

            for (Location loc : removeLocations) {
                req.removeLocation(loc.getQualifiedId());
            }
        }
    }
}
