package com.unitesk.requality.eclipse.handlers.requirement;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.browser.AddSelectionToRequirement;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;

public class CreateRequirementHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode node = getSelectedNode();
        String id = AddSelectionToRequirement.getNextId(node);
        Requirement newReq = (Requirement) node.createChild(id, "Requirement");
        newReq.saveAttributes();
        Activator.getDefault().setActiveNode(newReq, "Requirement");
        return null;
    }

    @Override
    public boolean isEnabled() {
        TreeNode node = getSelectedNode();
        if (node == null) {
            return false;
        }
        if (!node.getType().equals("Requirement")) {
            return false;
        }
        return true;
    }

    public synchronized TreeNode getSelectedNode() {
        ISelection selection = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getSelectionService()
                .getSelection();
        if (!(selection instanceof ITreeSelection)) {
            return null;
        }
        ITreeSelection selected = (ITreeSelection) selection;

        if (selected.isEmpty()) {
            return null;
        }

        Object obj = selected.getFirstElement();
        if (obj instanceof TreeNode) {
            TreeNode node = (TreeNode) obj;
            return node;
        } else if (obj instanceof IFile) {
            IFile f = (IFile) obj;
            TreeDB db = TreesTracker.getTracker().getTree(f.getProject());
            ResourceStorage rs = (ResourceStorage) db.getStorage();
            if (rs.getUUId(f) == null) {
                return null;
            }
            return db.getNode(rs.getUUId(f));
        }
        return null;
    }
}
