package com.unitesk.requality.eclipse.handlers.requirement;

import java.io.UnsupportedEncodingException;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.eclipse.views.RequalityCNF;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class RemoveRequirementHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode[] nodes = new TreeNode[1];
        nodes[0] = RequalityCNF.getSelectedNode();
        if (nodes.length == 1) {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Delete node",
                    "Delete selected " + nodes[0].getType())) {
                return null;
            }
        } else {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Delete nodes",
                    "Delete selected nodes")) {
                return null;
            }
        }
        for (TreeNode node : nodes) {
            if (node.getType().equals("Requirement")) {
                deleteLocations((Requirement) node);
            }
            node.delete();
        }
        return null;
    }

    // TODO: (low) check for crosslinks to locations
    private void deleteLocations(Requirement req) {
        for (Location loc : req.getLocations()) {
            if (loc == null) {
                // FIXME:
                continue;
            }
            try {
                Document doc = (Document) loc.getParent();
                if (doc == null)
                    return;
                IEditorReference eRef = Activator.findMarkEditorForDocument(doc
                        .getQualifiedId());
                if (eRef != null) {
                    if (eRef.getEditor(true) != null
                            && (eRef.getEditor(true) instanceof ReqMarker)) {
                        ReqMarker marker = (ReqMarker) eRef.getEditor(true);
                        marker.getBrowser().deleteLocation(loc.getId());
                        marker.saveDocument();
                    }
                } else {
                    doc.removeLocation(loc.getId());
                }
                doc.removeChild(loc);
                TreesTracker.getResourceByNode(doc).refreshLocal(
                        IResource.DEPTH_INFINITE, null);
            } catch (PartInitException e) {
                e.printStackTrace();
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
        for (int i = 0; i < req.sizeChildren(); i++) {
            TreeNode node = req.getChild(i);
            if (node instanceof Requirement) {
                Requirement r = (Requirement) node;
                deleteLocations(r);
            }
        }
    }
}
