package com.unitesk.requality.eclipse.views;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.navigator.CommonViewer;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.tools.ITreeStateListener;
import com.unitesk.requality.eclipse.tools.IUIStateChangeListener;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;

public class RequalityCNF extends CommonNavigator implements
        ITabbedPropertySheetPageContributor {
    public final static String ID = "com.unitesk.requality.views.RequalityExplorer";
    IResourceChangeListener resChangeListener = new IResourceChangeListener() {
        @Override
        public void resourceChanged(IResourceChangeEvent event) {
            try {
                if (event.getDelta() != null) {
                    event.getDelta().accept(new IResourceDeltaVisitor() {
                        @Override
                        public boolean visit(IResourceDelta delta)
                                throws CoreException {
                            if (delta.getResource() instanceof IProject) {
                                if (delta.getKind() == IResourceDelta.REMOVED) {
                                    refresh(null);
                                    return false;
                                }
                            }
                            return true;
                        }
                    });
                }
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }
    };

    ITreeChangeListener treeChangeListener = new ITreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
            getCommonViewer().refresh(oldParent);
            getCommonViewer().refresh(node.getParent());
        }

        @Override
        public void deleted(TreeNode node) {
            getCommonViewer().refresh(node.getParent());
        }

        @Override
        public void changed(TreeNode node) {
            // Hack for renaming
            refresh(node.getParent());
        }

        @Override
        public void added(TreeNode node) {
            if (node.getParent() == null) {
                getCommonViewer().refresh();
            } else {
                getCommonViewer().refresh(node.getParent());
            }
        }
    };

    ITreeStateListener treeStateListener = new ITreeStateListener() {
        @Override
        public void open(TreeDB db) {
            db.addListener(treeChangeListener);
            refresh(null);
        }

        @Override
        public void close(TreeDB db) {
            db.removeListener(treeChangeListener);
            refresh(null);
        }
    };

    final private IUIStateChangeListener uiListener = new IUIStateChangeListener() {
        @Override
        public void newActiveNode(TreeNode node) {
            if (node == null) {
                return;
            }
            Object obj = ((IStructuredSelection) getCommonViewer()
                    .getSelection()).getFirstElement();
            if (obj == null || obj.equals(node))
                return;
            Object parentElement = node.getParent();
            StructuredSelection element = new StructuredSelection(node);
            getCommonViewer().refresh(node.getParent());
            getCommonViewer().expandToLevel(parentElement, 1);
            selectReveal(element);
        }
    };

    final private ISelectionChangedListener treeSelection = new ISelectionChangedListener() {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            if (event.getSelection() instanceof IStructuredSelection) {
                Object obj = ((IStructuredSelection) event.getSelection())
                        .getFirstElement();
                if (obj instanceof TreeNode) {
                    TreeNode node = (TreeNode) obj;
                    if (node == null
                            || node.equals(Activator.getDefault()
                                    .getActiveNode(node.getType())))
                        return;
                    Activator.getDefault().setActiveNode(node, node.getType());
                }
            }
        }
    };

    public RequalityCNF() {
        super();
        for (TreeDB db : TreesTracker.getTracker().getTrees()) {
            db.addListener(treeChangeListener);
        }
        TreesTracker.getTracker().addListener(treeStateListener);
    }

    @Override
    public void dispose() {
        super.dispose();
        for (TreeDB db : TreesTracker.getTracker().getTrees()) {
            db.removeListener(treeChangeListener);
        }
        getCommonViewer().removePostSelectionChangedListener(treeSelection);
        TreesTracker.getTracker().removeListener(treeStateListener);
        Activator.getDefault().removeChangeListener(uiListener);
        ResourcesPlugin.getWorkspace().removeResourceChangeListener(
                resChangeListener);
    }

    public synchronized void refresh(final TreeNode treeNode) {
        if (!getCommonViewer().getControl().isDisposed()) {
            Display display = getCommonViewer().getControl().getDisplay();
            if (!display.isDisposed()) {
                display.syncExec(new Runnable() {
                    @Override
                    public void run() {
                        if (treeNode == null || (treeNode instanceof IProject)) {
                            getCommonViewer().setInput(
                                    ResourcesPlugin.getWorkspace().getRoot());
                            return;
                        }
                        getCommonViewer().refresh(treeNode);
                    }
                });
            }
        }
    }

    @Override
    public void createPartControl(Composite aParent) {
        super.createPartControl(aParent);
        Activator.getDefault().addChangeListener(uiListener);
        ResourcesPlugin.getWorkspace().addResourceChangeListener(
                resChangeListener);
    }

    // This is danger hack
    @Override
    protected CommonViewer createCommonViewerObject(Composite aParent) {
        CommonViewer cv = new CommonViewer(getViewSite().getId(), aParent,
                SWT.H_SCROLL | SWT.V_SCROLL);
        cv.addPostSelectionChangedListener(treeSelection);
        return cv;
    }

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequirementPropertyContributor";
    }

    @Override
    public Object getAdapter(@SuppressWarnings("rawtypes") Class adapter) {
        if (adapter == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(this);
        }
        return super.getAdapter(adapter);
    }

    public static synchronized TreeNode getSelectedNode() {
        ISelection selection = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getSelectionService()
                .getSelection();
        if (!(selection instanceof ITreeSelection)) {
            return null;
        }
        ITreeSelection selected = (ITreeSelection) selection;

        if (selected.isEmpty()) {
            return null;
        }

        Object obj = selected.getFirstElement();
        if (obj instanceof TreeNode) {
            TreeNode node = (TreeNode) obj;
            return node;
        } else if (obj instanceof IFile) {
            IFile f = (IFile) obj;
            TreeDB db = TreesTracker.getTracker().getTree(f.getProject());
            ResourceStorage rs = (ResourceStorage) db.getStorage();
            if (rs.getUUId(f) == null) {
                return null;
            }
            return db.getNode(rs.getUUId(f));
        }
        return null;
    }
}
