package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.eclipse.dialogs.EditAttrDialog;
import com.unitesk.requality.nodetypes.Requirement;

public class AttributesPanel {
    private Composite panel;
    private static final String columnNames[] = new String[] { "Name", "Value" };
    private Requirement req;
    private Map<String, String> innerAttr = new HashMap<String, String>();
    private TableViewer attribTable;
    private Button addAttrButton;
    private Button removeAttrButton;

    private Listener addAttrListener = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown && event.keyCode == SWT.INSERT)
                    || (event.type == SWT.Selection)) {
                EditAttrDialog addAttrDialog = new EditAttrDialog(
                        panel.getShell(), "", "", innerAttr);
                if (addAttrDialog.open() == EditAttrDialog.OK) {
                    innerAttr.put(addAttrDialog.getName(),
                            addAttrDialog.getValue());
                    attribTable.setInput(innerAttr);
                    attribTable.getControl().setFocus();
                    panel.notifyListeners(SWT.Modify, new Event());
                    if (req != null)
                    {
                        req.putAttribute(new Attribute(AttributeType.STRING, addAttrDialog.getName(), addAttrDialog.getValue()));
                        req.saveAttributes();
                    }
                }
            }
        }
    };

    private Listener removeAttrLink = new Listener() {
        @Override
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown && event.keyCode == SWT.DEL)
                    || (event.type == SWT.Selection)) {

                TableItem[] items = attribTable.getTable().getSelection();
                String mes = items.length <= 1 ? "Are you sure you want to remove selected attribute?"
                        : "Are you sure you want to remove selected attributes?";

                if ((items.length > 0)
                        && (MessageDialog.openQuestion(panel.getShell(),
                                "Attribute Editor", mes) == true)) {

                    for (int i = 0; i < items.length; i++) {
                        Object attrName = ((Object[]) items[i].getData())[0];
                        innerAttr.remove(attrName);
                        attribTable.refresh();
                        attribTable.getControl().setFocus();
                        panel.notifyListeners(SWT.Modify, new Event());
                        if (req != null)
                        {
                            req.removeAttribute((String)attrName);
                            req.saveAttributes();
                        }
                    }
                }
            }
        }
    };

    // TODO: refactor magic numbers
    public AttributesPanel(Composite parent, TabbedPropertySheetWidgetFactory wf) {
        panel = wf.createComposite(parent, SWT.NONE);
        panel.setLayout(new FormLayout());
//        GridData gd = new GridData(GridData.FILL_BOTH);
//        panel.setLayoutData(gd);
        FormData labelData = new FormData();
        labelData.top = new FormAttachment(0, 4);
        Label label = wf.createLabel(panel, "Attributes:", SWT.NONE);
        label.setLayoutData(labelData);

        FormData buttonsData = new FormData();
        buttonsData.left = new FormAttachment(label, 10);
        Composite attrsButtons = wf.createComposite(panel, SWT.NONE);
        buttonsData.height = 30;
        attrsButtons.setLayoutData(buttonsData);
        attrsButtons.setLayout(new GridLayout(2, false));
        
        GridData compactButton = new GridData();
        compactButton.heightHint = 28;
        compactButton.verticalIndent = -4;
        addAttrButton = wf.createButton(attrsButtons, "Add...", SWT.NONE);
        addAttrButton.setToolTipText("Add attribute");
        addAttrButton.addListener(SWT.Selection, addAttrListener);
        addAttrButton.setLayoutData(compactButton);

        removeAttrButton = wf.createButton(attrsButtons, "Remove...", SWT.NONE);
        removeAttrButton.setToolTipText("Remove attribute");
        removeAttrButton.addListener(SWT.Selection, removeAttrLink);
        removeAttrButton.setLayoutData(compactButton);

        attribTable = new TableViewer(panel, SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.BORDER | SWT.RESIZE);
        attribTable.setSorter(new ViewerSorter());
        FormData tableData = new FormData(0, 0);
        tableData.top = new FormAttachment(0, 30);
        tableData.left = new FormAttachment(0, 5);
        tableData.bottom = new FormAttachment(100, -5);
        tableData.right = new FormAttachment(100, -5);
        attribTable.getControl().setLayoutData(tableData);

        // Create columns
        for (String colName : columnNames) {
            TableViewerColumn column = new TableViewerColumn(attribTable,
                    SWT.H_SCROLL | SWT.V_SCROLL);
            column.getColumn().setText(colName);
            column.getColumn().setWidth(100);
            column.getColumn().setResizable(true);
            column.getColumn().setMoveable(true);
        }
        attribTable.setColumnProperties(columnNames);
        attribTable.getTable().setHeaderVisible(true);
        attribTable.getTable().setLinesVisible(true);
        // Editors
        CellEditor[] editors = new CellEditor[columnNames.length];
        editors[0] = new TextCellEditor(attribTable.getTable());
        editors[1] = new TextCellEditor(attribTable.getTable());
        attribTable.setCellEditors(editors);

        attribTable.setLabelProvider(new ITableLabelProvider() {
            @Override
            public void removeListener(ILabelProviderListener arg0) {
            }

            @Override
            public void addListener(ILabelProviderListener arg0) {
            }

            @Override
            public boolean isLabelProperty(Object arg0, String arg1) {
                return true;
            }

            @Override
            public void dispose() {
            }

            @Override
            public String getColumnText(Object element, int columnIndex) {
                Object[] row = (Object[]) element;
                return row[columnIndex].toString();
            }

            @Override
            public Image getColumnImage(Object element, int columnIndex) {
                return null;
            }
        });
        attribTable.setContentProvider(new IStructuredContentProvider() {
            @Override
            public Object[] getElements(Object input) {
                if (input instanceof Map<?, ?>) {
                    Map<?, ?> attrs = (Map<?, ?>) input;
                    ArrayList<Object[]> data = new ArrayList<Object[]>();
                    for (Object key : attrs.keySet()) {
                        data.add(new Object[] { key, attrs.get(key) });
                    }
                    return data.toArray(new Object[0]);
                }
                return null;
            }
            @Override
            public void dispose() {
            }
            @Override
            public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
            }
        });

        attribTable.addDoubleClickListener(new IDoubleClickListener() {
            @Override
            public void doubleClick(DoubleClickEvent event) {
                Object[] selection = (Object[]) ((IStructuredSelection) event.getSelection()).getFirstElement();
                EditAttrDialog addAttrDialog = new EditAttrDialog(getControl().getShell(),(String)selection[0], (String)selection[1],innerAttr);
                if (addAttrDialog.open() == EditAttrDialog.OK) {
                    innerAttr.put(addAttrDialog.getName(), addAttrDialog.getValue());
                    attribTable.setInput(innerAttr);
                    attribTable.getControl().setFocus();
                    panel.notifyListeners(SWT.Modify, new Event());
                    attribTable.refresh();
                }
            }
        });
    }

    public Control getControl() {
        return panel;
    }

    public void setRequirement(Requirement req) {
        this.req = req;
        innerAttr = new HashMap<String, String>();
        for (String key : req.getAttributeKeys()) {
            if (key.startsWith("_")) {
                continue;
            }
            innerAttr.put(key, req.getAttributeValue(key).toString());
        }
        attribTable.setInput(innerAttr);
    }

    public Map<String, String> getAttributes() {
        return innerAttr;
    }

}
