/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.views.properties.PropertySheet;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.dialogs.SelectNodeDialog;
import com.unitesk.requality.eclipse.dialogs.SelectReqDialog;
import com.unitesk.requality.eclipse.views.CNFLabelProvider;
import com.unitesk.requality.eclipse.views.RequalityCNF;
import com.unitesk.requality.nodetypes.Requirement;

public class BaseRequirementPanel {
    private Composite panel;
    private static final String EDIT_BASES = "Base requirements:";
    private static final String BUT_ADD = "Add...";
    private static final String BUT_REMOVE = "Remove...";
    static private String TITLE_ADD_BASE = "Add base requirement";
    static private String TITLE_REMOVE_BASE = "Remove base requirement";

    private TreeViewer basesTree;
    private Button addBaseButton;
    private Button removeBaseButton;

    private Menu basesMenu;
    static private int i = 0;
    private Requirement requirement;
    private List<String> baseReqIds = new ArrayList<String>();
    private IDoubleClickListener openBaseLink = new IDoubleClickListener() {

        @Override
        public void doubleClick(DoubleClickEvent event) {
            // TODO: move this logic to command handler
            IStructuredSelection selection = (IStructuredSelection) event
                    .getSelection();

            PropertySheet prop = (PropertySheet) PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage().getActivePart();

            RequalityCNF reqExpl = (RequalityCNF) PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage()
                    .findView(RequalityCNF.ID);
            reqExpl.getCommonViewer().setSelection(selection);
            PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                    .getActivePage().activate(reqExpl);
            IWorkbenchPage page = PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage();
            String viewId = "org.eclipse.ui.views.PropertySheet";
            IHandlerService handlerService = (IHandlerService) PlatformUI
                    .getWorkbench().getActiveWorkbenchWindow().getActivePage()
                    .getActivePart().getSite()
                    .getService(IHandlerService.class);
            try {
                handlerService.executeCommand(
                        "com.unitesk.requality.commands.OpenNode", null);
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (prop.isPinned()) {
                try {
                    page.showView(viewId, String.valueOf("req_prop_" + i),
                            IWorkbenchPage.VIEW_ACTIVATE);
                } catch (PartInitException e) {
                    e.printStackTrace();
                }
                i = i + 1;
            }
        }
    };

    public void setRequirement(Requirement req) {
        requirement = req;
        if (panel.isDisposed()) {
            return;
        }
        baseReqIds = new ArrayList<String>(req.getBaseNodes());
        basesTree.setInput(baseReqIds);
    }

    public BaseRequirementPanel(Composite parent,
            AdvancedPropertySection advancedPropertySection,
            TabbedPropertySheetPage propSheet) {
        TabbedPropertySheetWidgetFactory wf = propSheet.getWidgetFactory();
        panel = parent;
        panel.setLayout(new GridLayout());
        wf.createLabel(parent, EDIT_BASES);
        Composite baseCont = wf.createComposite(parent);
        baseCont.setLayout(new GridLayout(2, false));
        GridData gridData = new GridData(GridData.FILL_BOTH);
        baseCont.setLayoutData(gridData);
        basesTree = new TreeViewer(baseCont);
        GridData layoutData = new GridData(GridData.FILL_BOTH);
        layoutData.widthHint = 0;
        layoutData.heightHint = 0;
        basesTree.getControl().setLayoutData(layoutData);
        basesTree.setContentProvider(baseReqContentProvider);
        basesTree.setLabelProvider(new CNFLabelProvider());
        basesTree.addDoubleClickListener(openBaseLink);
        // basesTree.addListener( SWT.KeyDown, addBaseLink );
        // basesTree.addListener( SWT.KeyDown, openBaseLink );
        basesTree.getControl().addListener(SWT.KeyDown, removeBaseLink);

        // Создаем контекстное меню

        basesMenu = new Menu(basesTree.getControl());
        basesTree.getControl().setMenu(basesMenu);
        // basesMenu.addListener( SWT.Show, baseMenuLink );

        // openBaseItem= new MenuItem( basesMenu, SWT.NONE );
        // openBaseItem.setText( TITLE_OPEN_BASE );
        // openBaseItem.addListener( SWT.Selection, openBaseLink );

        // removeBaseItem= new MenuItem( basesMenu, SWT.NONE );
        // removeBaseItem.setText( TITLE_REMOVE_BASE );
        // removeBaseItem.addListener( SWT.Selection, removeBaseLink );

        Composite parsButtons = wf.createComposite(baseCont);
        parsButtons.setLayout(new GridLayout());
        parsButtons.setLayoutData(new GridData(GridData.FILL_VERTICAL));

        // Создаем кнопку "Добавление"
        addBaseButton = wf.createButton(parsButtons, BUT_ADD, SWT.PUSH);
        addBaseButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        addBaseButton.setToolTipText(TITLE_ADD_BASE);

        // Создаем кнопку "Удаление"
        removeBaseButton = wf.createButton(parsButtons, BUT_REMOVE, SWT.PUSH);
        removeBaseButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        removeBaseButton.setToolTipText(TITLE_REMOVE_BASE);

        // Реакция на нажатие соответствующей кнопки
        addBaseButton.addListener(SWT.Selection, addBaseLink);
        removeBaseButton.addListener(SWT.Selection, removeBaseLink);

        // Реакция на нажатие клавиши "вниз"
        // addBaseButton.addListener( SWT.KeyDown, buttonLink );
        // removeBaseButton.addListener( SWT.KeyDown, buttonLink );
        initModifyListener();
    }

    protected void updateUI() {
        removeBaseButton.setEnabled(!basesTree.getSelection().isEmpty());
    }

    protected void initModifyListener() {
        updateUI();
        basesTree.addSelectionChangedListener(new ISelectionChangedListener() {
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                updateUI();
            }
        });
    }

    public boolean isChanged() {
        return !requirement.getBaseNodes().equals(baseReqIds);
    }

    public List<String> getBaseReqIds() {
        return baseReqIds;
    }

    private Listener addBaseLink = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown) && (event.keyCode == SWT.INSERT)
                    || (event.type == SWT.Selection)) {
                SelectNodeDialog selDialog = new SelectReqDialog(
                        panel.getShell(), requirement.getTreeDB());
                if (selDialog.open() == SelectNodeDialog.OK) {
                    Requirement selected = (Requirement) selDialog.getResult();
                    if (selected != null) {
                        if (baseReqIds.contains(selected.getUUId())) {
                            MessageDialog.openInformation(panel.getShell(),
                                    "Duplicate base requirement",
                                    "Base requirement is already in list");
                            return;
                        }

                        baseReqIds.add(selected.getQualifiedId());
                        basesTree.refresh();
                        panel.notifyListeners(SWT.Modify, new Event());
                        requirement.addBaseNode(selected.getQualifiedId());
                        requirement.saveAttributes();
                    }
                }
            }
        }
    };

    Listener removeBaseLink = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown) && (event.keyCode == SWT.DEL)
                    || (event.type == SWT.Selection)) {
                if (basesTree.getSelection() instanceof ITreeSelection) {
                    ITreeSelection treeSel = (ITreeSelection) basesTree
                            .getSelection();
                    if (treeSel.getFirstElement() instanceof Requirement) {
                        Requirement selReq = (Requirement) treeSel
                                .getFirstElement();
                        baseReqIds.remove(selReq.getQualifiedId());
                        panel.notifyListeners(SWT.Modify, new Event());
                        basesTree.refresh();
                        basesTree.getControl().setFocus();
                        requirement.removeBaseNode(selReq.getQualifiedId());
                        requirement.saveAttributes();
                    }
                }
            }
        }
    };

    private ITreeContentProvider baseReqContentProvider = new ITreeContentProvider() {

        @Override
        public void inputChanged(Viewer arg0, Object arg1, Object arg2) {

        }

        @Override
        public void dispose() {

        }

        @Override
        public Object[] getElements(Object element) {
            return getChildren(element);
        }

        @Override
        public boolean hasChildren(Object element) {
            return false;
        }

        @Override
        public Object getParent(Object element) {
            return null;
        }

        @SuppressWarnings("unchecked")
        @Override
        public Object[] getChildren(Object parentElement) {
            if (parentElement instanceof List<?>) {
                List<String> listIds = (List<String>) parentElement;
                List<TreeNode> result = new ArrayList<TreeNode>();
                for (String baseId : listIds) {
                    result.add(requirement.getTreeDB().getNode(baseId));
                }

                return result.toArray(new TreeNode[0]);
            }
            return null;
        }
    };
}
