package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.tools.ITreeStateListener;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;

public abstract class RequirementPropertiesTab extends AbstractPropertySection{
    protected Requirement req;

    ITreeChangeListener treeChangeListener = new ITreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
            if (req != null)
                setRequirement(req);
        }

        @Override
        public void deleted(TreeNode node) {
            if (req != null)
                setRequirement(req);
        }

        @Override
        public void changed(TreeNode node) {
            if (req != null)
                setRequirement(req);
        }

        @Override
        public void added(TreeNode node) {
            if (req != null)
                setRequirement(req);
        }
    };

    ITreeStateListener treeStateListener = new ITreeStateListener() {
        @Override
        public void open(TreeDB db) {
            db.addListener(treeChangeListener);
            if (req != null)
                setRequirement(req);
        }

        @Override
        public void close(TreeDB db) {
            db.removeListener(treeChangeListener);
            if (req != null)
                setRequirement(req);
        }
    };

    public RequirementPropertiesTab() {
        super();
        for (TreeDB db : TreesTracker.getTracker().getTrees()) {
            db.addListener(treeChangeListener);
        }
        TreesTracker.getTracker().addListener(treeStateListener);
    }

    @Override
    public void dispose() {
        for (TreeDB db : TreesTracker.getTracker().getTrees()) {
            db.removeListener(treeChangeListener);
        }
        TreesTracker.getTracker().removeListener(treeStateListener);
        super.dispose();
    }

    abstract protected void setRequirement(Requirement req);

    @Override
    public void setInput(IWorkbenchPart part, ISelection selection) {
        super.setInput(part, selection);
        Assert.isTrue(selection instanceof IStructuredSelection);
        IStructuredSelection sel = (IStructuredSelection) selection;
        if (sel.getFirstElement() instanceof Requirement) {
            req = (Requirement) sel.getFirstElement();
            setRequirement(req);
        }
    }


}
