package com.unitesk.requality.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Map;

import org.junit.Test;

import com.unitesk.requality.core.MemoryStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.documents.DocumentImporter;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.Converter;

public class LegacyImport {
    private String getInput(String clazz) {
        return "<html><head></head><body><p class=\""+clazz+"\" id=\"l64a.09;test.01.02\">thread-safe.</p></body></html>";
    }
    private String getResult(String locId) {
        StringBuffer sb = new StringBuffer();
        sb.append("<html xmlns=\"http://www.w3.org/1999/xhtml\">");
        sb.append("<head></head>");
        sb.append("<body>");
        sb.append("<p><span class=\"requality_text id_" + locId + "\">");
        sb.append("<a id=\"id_" + locId+ "\" name=\"" + locId + "\" class=\"requality_id\"></a>");
        sb.append("thread-safe.");
        sb.append("</span></p>");
        sb.append("</body></html>");
        return sb.toString();
    }

    @Test
    public void importReq() {
        TreeDB db = new TreeDB(new MemoryStorage());
        db.addNodeType("Requirement", Requirement.class);
        db.addNodeType("DocFolder", DocFolder.class);
        db.addNodeType("Document", Document.class);
        db.addNodeType("Location", Location.class);
        Requirement rootReq = (Requirement) db.getRootNode().createChild(
                "Requirements", "Requirement");

        try {
            StringWriter tmpOut = new StringWriter();
            StringWriter out = new StringWriter();
            DocumentImporter.cleanHtml(new StringReader(getInput("req")), tmpOut);
            Converter c = new Converter();
            Map<String, String[]> newReqs = c.convert(new StringReader(tmpOut.toString()), out);
            out.flush();
            assertEquals(2, newReqs.size());
            assertTrue(newReqs.containsKey("/l64a/09"));
            assertTrue(newReqs.containsKey("/test/01/02"));
            assertEquals(1, newReqs.get("/l64a/09").length);
            assertEquals(getResult(newReqs.get("/l64a/09")[0]), out.toString());
            assertEquals("<p>thread-safe.</p>", c.getLocationDescription(newReqs.get("/l64a/09")[0]));
            String docId = "/Documents/doc1";
            String locQId = docId+"/"+newReqs.get("/l64a/09")[0];
            db.getRootNode().createChild("Documents","DocFolder").createChild("doc1","Document");
            c.store(newReqs, docId, db);
            assertEquals(2, rootReq.sizeChildren());
            assertNotNull(rootReq.findChild("l64a"));
            Requirement oneChild = (Requirement) rootReq.findChild("test").findChild("01").findChild("02");
            assertNotNull(oneChild);
            assertEquals(1, oneChild.getLocationQIds().size());
            assertTrue(oneChild.getLocationQIds().contains(locQId));
            assertEquals(1, oneChild.getLocations().size());
            assertTrue(oneChild.getLocations().contains(db.getNode(locQId)));
        } catch (IOException e) {
            e.printStackTrace();
            fail();
        }
    }

    @Test
    public void convert() {
        try {
            StringWriter tmpOut = new StringWriter();
            StringWriter out = new StringWriter();
            DocumentImporter.cleanHtml(new StringReader("<p class=\"req\" id=\"req1\">l1</p>\n<p class=\"req\" id=\"req1\">l2</p>"), tmpOut);
            Converter c = new Converter();
            Map<String, String[]> newReqs = c.convert(new StringReader(tmpOut.toString()), out);
            out.flush();
            assertEquals(getResultWrap(newReqs.get("/req1")[0],newReqs.get("/req1")[1]), out.toString());
            assertEquals("<p>l1</p>", c.getLocationDescription(newReqs.get("/req1")[0]));
            assertEquals("<p>l2</p>", c.getLocationDescription(newReqs.get("/req1")[1]));
        } catch (IOException e) {
            e.printStackTrace();
            fail();
        }
    }
    private String getResultWrap(String locId1, String locId2) {
        StringBuffer sb = new StringBuffer();
        sb.append("<html xmlns=\"http://www.w3.org/1999/xhtml\">");
        sb.append("<head></head><body><p>");
        sb.append("<span class=\"requality_text id_"+locId1+"\">");
        sb.append("<a id=\"id_"+locId1+"\" name=\""+locId1+"\" class=\"requality_id\"></a>");
        sb.append("l1");
        sb.append("</span></p>");
        sb.append("<p><span class=\"requality_text id_"+locId2+"\">");
        sb.append("<a id=\"id_"+locId2+"\" name=\""+locId2+"\" class=\"requality_id\"></a>");
        sb.append("l2");
        sb.append("</span></p>");
        sb.append("</body></html>");
        return sb.toString();
    }

    @Test
    public void convertInnerTags() {
        try {
            StringWriter tmpOut = new StringWriter();
            StringWriter out = new StringWriter();
            DocumentImporter.cleanHtml(new StringReader("<p class=\"req\" id=\"req1\"><i>l</i>1</p>"), tmpOut);
            Converter c = new Converter();
            Map<String, String[]> newReqs = c.convert(new StringReader(tmpOut.toString()), out);
            out.flush();
            assertEquals(getResultWrapInnerTags(newReqs.get("/req1")[0]), out.toString());
            assertEquals(getLocDescInnerTags(), c.getLocationDescription(newReqs.get("/req1")[0]));
            // TODO: location description
        } catch (IOException e) {
            e.printStackTrace();
            fail();
        }
    }

    private String getResultWrapInnerTags(String locId1) {
        StringBuffer sb = new StringBuffer();
        sb.append("<html xmlns=\"http://www.w3.org/1999/xhtml\">");
        sb.append("<head></head><body><p>");
        sb.append("<i>");
        sb.append("<span class=\"requality_text id_"+locId1+"\">");
        sb.append("<a id=\"id_"+locId1+"\" name=\""+locId1+"\" class=\"requality_id\"></a>");
        sb.append("l");
        sb.append("</span>");
        sb.append("</i>");
        sb.append("<span class=\"requality_text id_"+locId1+"\">");
        sb.append("1");
        sb.append("</span>");
        sb.append("</p>");
        sb.append("</body></html>");
        return sb.toString();
    }
    private String getLocDescInnerTags() {
        StringBuffer sb = new StringBuffer();
        sb.append("<p><i>");
        sb.append("l");
        sb.append("</i>");
        sb.append("1");
        sb.append("</p>");
        return sb.toString();
    }}
