package com.unitesk.requality.tests.merge.acceptance;

import java.io.File;
import java.io.FilenameFilter;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import com.unitesk.requality.merge.Location;
import com.unitesk.requality.merge.Chunk;
import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.MergeProcessorAccessor;
import com.unitesk.requality.tests.merge.Pair;
import com.unitesk.requality.tests.merge.Utils;

public class LSBTest extends TestCase {

    private static final String markedPosixDir = "./test-data/merge/lsb/v3/";
    private static final String unmarkedPosixDir = "./test-data/merge/lsb/v4/";
    private static final String mergedDir = "./test-data/merge/lsb/result/";
    private List<Pair<File, File>> testData;

    public void setUp() {
	testData = new ArrayList<Pair<File, File>>();
	prepareData();
	if (testData.size() == 0)
	    fail("No test data for tests");
	if (!new File(mergedDir).exists())
	    new File(mergedDir).mkdir();
	mergeData();
	System.out.println("Merge complete. Verifying...");
    }

    public void tearDown() {
	File[] mergedFiles = new File(mergedDir).listFiles();
	for (File f : mergedFiles)
	    f.delete();
	new File(mergedDir).delete();
    }

    private void prepareData() {
	File sourceDir = new File(markedPosixDir);
	File targetDir = new File(unmarkedPosixDir);
	FilenameFilter filter = new FilenameFilter() {
	    public boolean accept(File dir, String name) {
		if (name.endsWith(".xhtml"))
		    return true;
		return false;
	    }
	};
	File[] sourceFiles = sourceDir.listFiles(filter);
	File[] targetFiles = targetDir.listFiles(filter);
	if (sourceFiles != null && targetFiles != null)
	    for (File src : sourceFiles) {
		for (File trg : targetFiles) {
		    if (src.getName().equals(trg.getName())) {
			Pair<File, File> docs = new Pair<File, File>(src, trg);
			testData.add(docs);
			break;
		    }
		}
	    }
    }

    private void mergeData() {
	int counter = 0;
	for (Pair<File, File> docs : testData) {
	    counter++;
	    Document src = Utils.parseXml(docs.getKey().getPath());
	    Document trg = Utils.parseXml(docs.getValue().getPath());
	    MergeProcessor.getInstance().merge(src, trg);
	    Utils.writeXmlFile(trg, mergedDir + docs.getValue().getName());
	    System.out.println("Merged " + mergedDir + docs.getKey().getName()
		    + " (" + counter + " of " + testData.size() + ")");
	}
    }

    public void testLsbMerge() {
	int fail_count = 0;
	int total_locations = 0;
	int total_locations_merged = 0;
	int counter = 0;
	for (Pair<File, File> docs : testData) {
	    counter++;
	    Document source = Utils.parseXml(docs.getKey().getPath());
	    Document mergeResult = Utils.parseXml(mergedDir
		    + docs.getKey().getName());
	    MergeProcessorAccessor mpa = new MergeProcessorAccessor(
		    MergeProcessor.getInstance());
	    try {
		List<Location> locationsRef = mpa.lookupLocations(new Tree(
			source));
		List<Location> locationsMerged = mpa.lookupLocations(new Tree(
			mergeResult));
		total_locations += locationsRef.size();
		total_locations_merged += locationsMerged.size();
		List<Location> successfullyMerged = new ArrayList<Location>();
		for (Location locRef : locationsRef) {
		    String idRef = locRef.getLocationId();
		    for (Location locMerged : locationsMerged) {
			if (locMerged.getLocationId().equals(idRef)) {
			    Iterator<Chunk> i = locRef.getChunks()
				    .listIterator();
			    Iterator<Chunk> j = locMerged.getChunks()
				    .listIterator();
			    int local_fail_count = 0;
			    while (i.hasNext() && j.hasNext()) {
				Chunk ref = i.next();
				Chunk res = j.next();
				if (!ref.getChunkText().equals(
					res.getChunkText())) {
				    local_fail_count++;
				    System.out
					    .println("Failed to merge properly "
						    + ref
						    + " in "
						    + docs.getKey().getName());
				}
			    }
			    if (local_fail_count == 0) {
				successfullyMerged.add(locMerged);
				break;
			    } else
				fail_count += local_fail_count;
			}
		    }
		}
	    } catch (InvocationTargetException e) {
		e.printStackTrace();
		fail("Test case is broken");
	    }
	}
	System.out.println("Total locations in source documents: "
		+ total_locations);
	System.out.println("Total locations merged: " + total_locations_merged
		+ " - " + (float) total_locations_merged / total_locations
		* 100 + "%");
	System.out.println("Merge failed " + fail_count + " times");
	assertEquals("Fail counter isn't zero", 0, fail_count);
    }
}
