package com.unitesk.requality.tests.merge.sanity;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import com.unitesk.requality.merge.Chunk;
import com.unitesk.requality.merge.Location;
//import com.unitesk.requality.merge.Location.TraceFailedException;
import com.unitesk.requality.merge.Location.TraceFailedException;
import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.merge.Node;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.LocationAccessor;
import com.unitesk.requality.tests.merge.MergeProcessorAccessor;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.Pair;

/*
 * Location ids:
 * 1 - basic location
 * 2 - location with several chunks
 * 3 - text-splitting location
 * 4 - missing location
 * 5 - location with edited text
 * 6 - location with changed context
 */

public class LocationTest extends TestCase {

    private MergeProcessorAccessor mpa;
    private Map<String, Pair<Document, Document>> testData;

    public void setUp() {
	MergeProcessor mp = MergeProcessor.getInstance();
	mpa = new MergeProcessorAccessor(mp);
	testData = TestDataHelper.getInstance().getSanityData();
    }

    public void testAddChunk() {
	try {
	    Document v1 = testData.get("Sanity").getKey();
	    Tree t = new Tree(v1);
	    List<Location> locations = mpa.lookupLocations(t);
	    assertEquals(1, locations.get(0).getChunks().size());
	    assertEquals(2, locations.get(1).getChunks().size());
	    assertEquals(1, locations.get(2).getChunks().size());
	} catch (InvocationTargetException e) {
	    fail("Broken test case");
	}
    }

    public void testTraceLocation() {
	try {
	    Document v1 = testData.get("Sanity").getKey();
	    Document v2 = testData.get("Sanity").getValue();
	    Tree t1 = new Tree(v1);
	    Tree t2 = new Tree(v2);
	    List<Location> locations = mpa.lookupLocations(t1);
	    for(Location loc : locations){
		String id = loc.getLocationId();
		if(id.equals("1")){
		    Location newLoc = loc.traceLocation(t2);
		    assertNotNull(newLoc);
		    assertEquals(Location.Status.TRACED, loc.getStatus());
		} else if (id.equals("2")){
		    Location newLoc = loc.traceLocation(t2);
		    assertNotNull(newLoc);
		    assertEquals(Location.Status.TRACED, loc.getStatus());
		} else if (id.equals("3")){
		    Location newLoc = loc.traceLocation(t2);
		    assertNotNull(newLoc);
		    assertEquals(Location.Status.TRACED, loc.getStatus());
		} else if (id.equals("4")){
		    try {
			loc.traceLocation(t2);
			fail("Exception expected");
		    } catch (TraceFailedException e) {
			assertEquals(Location.Status.MISSING, loc.getStatus());
		    }
		} else if (id.equals("5")){
		    try {
			loc.traceLocation(t2);
			fail("Exception expected");
		    } catch (TraceFailedException e) {
			assertEquals(Location.Status.CHANGED, loc.getStatus());
		    }
		} else if (id.equals("6")){
		    try {
			loc.traceLocation(t2);
			fail("Exception expected");
		    } catch (TraceFailedException e) {
			assertEquals(Location.Status.MOVED, loc.getStatus());
		    }
		}
	    }
	} catch (InvocationTargetException e) {
	    fail("Broken test case");
	} /*catch (TraceFailedException e) {
	    e.printStackTrace();
	    fail("Invalid test data");
	}*/ catch (TraceFailedException e) {
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	    fail();
	}
    }

    public void testTraceParent() {
	try {
	    Document v1 = testData.get("Sanity").getKey();
	    Document v2 = testData.get("Sanity").getValue();
	    Tree t1 = new Tree(v1);
	    Tree t2 = new Tree(v2);
	    List<Location> locations = mpa.lookupLocations(t1);
	    for(Location loc : locations){
		String id = loc.getLocationId();
		if(id.equals("1")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    assertTrue(parent.isEqual(loc.getCommonParent()));
		} else if (id.equals("2")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    assertTrue(parent.isEqual(loc.getCommonParent()));
		} else if (id.equals("3")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    assertTrue(parent.isEqual(loc.getCommonParent()));
		} else if (id.equals("4")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNull(parent);
		} else if (id.equals("5")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    assertTrue(parent.isEqual(loc.getCommonParent()));
		} else if (id.equals("6")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    assertFalse(parent.isEqual(loc.getCommonParent()));
		}
	    }
	} catch (InvocationTargetException e) {
	    fail("Broken test case");
	}
    }
    
    public void testTraceChunk() {
	try {
	    Document v1 = testData.get("Sanity").getKey();
	    Document v2 = testData.get("Sanity").getValue();
	    Tree t1 = new Tree(v1);
	    Tree t2 = new Tree(v2);
	    List<Location> locations = mpa.lookupLocations(t1);
	    for(Location loc : locations){
		String id = loc.getLocationId();
		if(id.equals("1")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    for(Chunk chunk : loc.getChunks()) {
			Node anchor = new LocationAccessor(loc).traceChunk(chunk, parent, t2);
			assertNotNull(anchor);
			assertEquals(chunk.getChunkText(), anchor.getText().trim());
		    }
		} else if (id.equals("2")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    for(Chunk chunk : loc.getChunks()) {
			Node anchor = new LocationAccessor(loc).traceChunk(chunk, parent, t2);
			assertNotNull(anchor);
			assertTrue(anchor.getText().contains(chunk.getChunkText()));
		    }
		} else if (id.equals("3")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    for(Chunk chunk : loc.getChunks()) {
			Node anchor = new LocationAccessor(loc).traceChunk(chunk, parent, t2);
			assertNotNull(anchor);
			assertTrue(anchor.getText().contains(chunk.getChunkText()));
		    }
		} else if (id.equals("5")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    for(Chunk chunk : loc.getChunks()) {
			Node anchor = new LocationAccessor(loc).traceChunk(chunk, parent, t2);
			assertNull(anchor);
		    }
		} else if (id.equals("6")){
		    Node parent = new LocationAccessor(loc).traceParent(t2);
		    assertNotNull(parent);
		    for(Chunk chunk : loc.getChunks()) {
			Node anchor = new LocationAccessor(loc).traceChunk(chunk, parent, t2);
			assertNotNull(anchor);
		    }
		}
	    }
	} catch (InvocationTargetException e) {
	    fail("Broken test case");
	}
    }
    
    public void testGetCommonParent() {
	try {
	    Document v1 = testData.get("Sanity").getKey();
	    Tree t1 = new Tree(v1);
	    List<Location> locations = mpa.lookupLocations(t1);
	    for (Location loc : locations) {
		assertEquals(loc.getChunks().get(0).getSpanNode().getParent(),
			loc.getCommonParent());
	    }
	} catch (InvocationTargetException e) {
	    fail("Broken test case");
	}
    }
}
