package com.unitesk.requality.tests.merge.sanity;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;

import org.w3c.dom.Document;

import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.Pair;
import com.unitesk.requality.tests.merge.Utils;

import junit.framework.*;

public class MergeTest extends TestCase {

    private MergeProcessor mp;
    private Map<String, Pair<Document, Document>> testData;

    public void setUp() {
	TestDataHelper.update();
	testData = TestDataHelper.getInstance().getSanityData();
	mp = MergeProcessor.getInstance();
    }
    
    public void testMerge1() {
	Document v1 = testData.get("Sanity").getKey();
	Document v2 = testData.get("Sanity").getValue();
	mp.merge(v1, v2);
	Tree t1 = new Tree(v1);
	Tree result = new Tree(v2);
	assertTrue("Trees aren't equal", t1.isEqual(result));
    }
    
    public void testMerge2() {
	try {
	    Tree t1 = new Tree(testData.get("Sanity").getKey());
	    File v1 = new File(new URI(testData.get("Sanity").getKey().getDocumentURI()));
	    File v2 = new File(new URI(testData.get("Sanity").getValue().getDocumentURI()));
	    File result = new File("/tmp/result.xml");
	    Reader r_v1 = new FileReader(v1);
	    Reader r_v2 = new FileReader(v2);
	    Writer w_res = new FileWriter(result);
	    mp.merge(r_v1, r_v2, w_res);
	    Document res = Utils.parseXml(result.getAbsolutePath());
	    assertTrue("Trees aren't equal", t1.isEqual(new Tree(res)));
	} catch (FileNotFoundException e) {
	    e.printStackTrace();
	    fail("Broken test case");
	} catch (IOException e) {
	    e.printStackTrace();
	    fail("Broken test case");
	} catch (URISyntaxException e) {
	    e.printStackTrace();
	    fail("Broken test case");
	}
    }
}
